@@ -1,7 +1,7 @@
 #==============================================================================
 # Contains some Tk option database settings.
 #
-# Copyright (c) 2004-2018  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2004-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -22,7 +22,7 @@ if {[string compare $winSys "x11"] == 0} {
     #
     # Create the font TkDefaultFont if not yet present
     #
-    catch {font create TkDefaultFont -family Helvetica -size -12}
+    catch {font create TkDefaultFont -family Helvetica -size 9}
 
     option add *Font			TkDefaultFont
     option add *selectBackground	#5294e2
