@@ -1,7 +1,7 @@
 #==============================================================================
 # Contains some Tk option database settings.
 #
-# Copyright (c) 2010-2018  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2010-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -21,9 +21,10 @@ if {[catch {tk windowingsystem} winSys] != 0} {
 switch $winSys {
     x11 {
 	#
-	# Create the font TkDefaultFont if not yet present
+	# Create the fonts TkDefaultFont and TkFixedFont if not yet present
 	#
-	catch {font create TkDefaultFont -family Helvetica -size -12}
+	catch {font create TkDefaultFont -family Helvetica -size 9}
+	catch {font create TkFixedFont -family Courier -size 9}
 
 	option add *Font		TkDefaultFont
 	option add *selectBackground	#5294e2
