@@ -7,7 +7,7 @@
 #   - Private procedures related to tile themes
 #   - Private procedures related to global KDE configuration options
 #
-# Copyright (c) 2006-2018  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2006-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -15,6 +15,15 @@
 # ========================================
 #
 
+#------------------------------------------------------------------------------
+# mentry::getCurrentTheme
+#
+# Returns the current tile theme.
+#------------------------------------------------------------------------------
+proc mentry::getCurrentTheme {} {
+    return [mwutil::currentTheme]
+}
+
 #------------------------------------------------------------------------------
 # mentry::setThemeDefaults
 #
@@ -22,18 +31,67 @@
 # mentry configuration options and updates the array configSpecs.
 #------------------------------------------------------------------------------
 proc mentry::setThemeDefaults {} {
+    set currentTheme [mwutil::currentTheme]
+    variable isAwTheme \
+	[llength [info commands ::ttk::theme::${currentTheme}::setTextColors]]
+
     variable themeDefaults
     if {[info exists themeDefaults(-readonlybackground)]} {
 	unset themeDefaults(-readonlybackground)
     }
 
-    if {[catch {[getCurrentTheme]Theme}] != 0} {
+    if {[catch {${currentTheme}Theme}] != 0} {
 	#
 	# Fall back to the "default" theme (which is the root of all
 	# themes) and then override the options set by the current one
 	#
 	defaultTheme
-	array set themeDefaults [style configure .]
+	array set themeDefaults [styleConfig .]
+
+	if {[set bg [styleConfig TEntry -fieldbackground]] eq ""} {
+	    set bg [styleConfig . -fieldbackground]
+	}
+	if {$bg ne ""} {
+	    set themeDefaults(-background) $bg
+	}
+
+	if {[set fg [styleConfig TEntry -foreground]] eq ""} {
+	    set fg [styleConfig . -foreground]
+	}
+	if {$fg ne ""} {
+	    set themeDefaults(-foreground) $fg
+	}
+	set themeDefaults(-foreground,background) $themeDefaults(-foreground)
+
+	set disabledBg ""
+	array set arr [style map TEntry -fieldbackground]
+	if {[info exists arr(disabled)]} {
+	    set disabledBg $arr(disabled)
+	} else {
+	    array set arr [style map . -fieldbackground]
+	    if {[info exists arr(disabled)]} {
+		set disabledBg $arr(disabled)
+	    }
+	}
+	set themeDefaults(-disabledbackground) $disabledBg	;# may be ""
+
+	set disabledFg ""
+	unset arr
+	array set arr [style map TEntry -foreground]
+	if {[info exists arr(disabled)]} {
+	    set disabledFg $arr(disabled)
+	} else {
+	    array set arr [style map . -foreground]
+	    if {[info exists arr(disabled)]} {
+		set disabledFg $arr(disabled)
+	    }
+	}
+	if {$disabledFg ne ""} {
+	    set themeDefaults(-disabledforeground) $disabledFg
+	}
+
+	set themeDefaults(-borderwidth) 2			;# just a guess
+	set themeDefaults(-labelpady)   {2 2}			;# just a guess
     }
 
     if {![info exists themeDefaults(-readonlybackground)]} {
@@ -80,19 +138,49 @@ proc mentry::altTheme {} {
 # mentry::aquaTheme
 #------------------------------------------------------------------------------
 proc mentry::aquaTheme {} {
+    variable newAquaSupport
     variable themeDefaults
-    array set themeDefaults [list \
-	-background		systemWindowBody \
-	-disabledbackground	"" \
-	-foreground		systemModelessDialogActiveText \
-	-foreground,background	systemModelessDialogInactiveText \
-	-disabledforeground	systemModelessDialogInactiveText \
-	-selectbackground	systemHighlight \
-	-selectforeground	systemModelessDialogActiveText \
-	-selectborderwidth	0 \
-	-borderwidth		4 \
-	-labelpady		{4 4} \
-    ]
+    if {$newAquaSupport} {
+	variable extendedAquaSupport
+	if {[tk::unsupported::MacWindowStyle isdark .]} {
+	    set background [expr {$extendedAquaSupport ? "#3a3a3a" : "#4b4b4b"}]
+	} else {
+	    set background #ffffff
+	}
+
+	if {$extendedAquaSupport} {
+	    scan $::tcl_platform(osVersion) "%d" majorOSVersion
+	    set labelPadY [expr {$majorOSVersion >= 18 ? {4 7} : {4 5}}]
+	} else {
+	    set labelPadY {4 7}
+	}
+
+	array set themeDefaults [list \
+	    -background			$background \
+	    -disabledbackground		"" \
+	    -foreground			systemTextColor \
+	    -foreground,background	systemTextColor \
+	    -disabledforeground		systemDisabledControlTextColor \
+	    -selectbackground		systemSelectedTextBackgroundColor \
+	    -selectforeground		systemSelectedTextColor \
+	    -selectborderwidth		0 \
+	    -borderwidth		4 \
+	    -labelpady			$labelPadY \
+	]
+    } else {
+	array set themeDefaults [list \
+	    -background			systemWindowBody \
+	    -disabledbackground		"" \
+	    -foreground			systemModelessDialogActiveText \
+	    -foreground,background	systemModelessDialogInactiveText \
+	    -disabledforeground		systemModelessDialogInactiveText \
+	    -selectbackground		systemHighlight \
+	    -selectforeground		systemModelessDialogActiveText \
+	    -selectborderwidth		0 \
+	    -borderwidth		4 \
+	    -labelpady			{4 4} \
+	]
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -148,10 +236,118 @@ proc mentry::ArcTheme {} {
 	-selectforeground	#ffffff \
 	-selectborderwidth	0 \
 	-borderwidth		3 \
-	-labelpady		{1 1} \
+	-labelpady		{3 3} \
     ]
 }
 
+#------------------------------------------------------------------------------
+# mentry::arcTheme
+#------------------------------------------------------------------------------
+proc mentry::arcTheme {} {
+    variable isAwTheme
+    if {$isAwTheme} {
+	awarcTheme 
+    } else {
+	return -code error			;# handled by setThemeDefaults
+    }
+}
+
+#------------------------------------------------------------------------------
+# mentry::awarcTheme
+#------------------------------------------------------------------------------
+proc mentry::awarcTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	3 \
+	-labelpady	{3 3} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::awblackTheme
+#------------------------------------------------------------------------------
+proc mentry::awblackTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	2 \
+	-labelpady	{2 2} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::awbreezeTheme
+#------------------------------------------------------------------------------
+proc mentry::awbreezeTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	3 \
+	-labelpady	{3 3} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::awclearlooksTheme
+#------------------------------------------------------------------------------
+proc mentry::awclearlooksTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	2 \
+	-labelpady	{2 2} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::awdarkTheme
+#------------------------------------------------------------------------------
+proc mentry::awdarkTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	2 \
+	-labelpady	{2 2} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::awlightTheme
+#------------------------------------------------------------------------------
+proc mentry::awlightTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	2 \
+	-labelpady	{2 2} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::awwinxpblueTheme
+#------------------------------------------------------------------------------
+proc mentry::awwinxpblueTheme {} {
+    awTheme 
+    variable themeDefaults
+    array set themeDefaults [list \
+	-borderwidth	2 \
+	-labelpady	{2 2} \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# mentry::blackTheme
+#------------------------------------------------------------------------------
+proc mentry::blackTheme {} {
+    variable isAwTheme
+    if {$isAwTheme} {
+	awblackTheme 
+    } else {
+	return -code error			;# handled by setThemeDefaults
+    }
+}
+
 #------------------------------------------------------------------------------
 # mentry::blueTheme
 #------------------------------------------------------------------------------
@@ -166,11 +362,23 @@ proc mentry::blueTheme {} {
 	-selectbackground	#ffff33 \
 	-selectforeground	#000000 \
 	-selectborderwidth	1 \
-	-borderwidth		1 \
-	-labelpady		{1 1} \
+	-borderwidth		2 \
+	-labelpady		{2 2} \
     ]
 }
 
+#------------------------------------------------------------------------------
+# mentry::breezeTheme
+#------------------------------------------------------------------------------
+proc mentry::breezeTheme {} {
+    variable isAwTheme
+    if {$isAwTheme} {
+	awbreezeTheme 
+    } else {
+	return -code error			;# handled by setThemeDefaults
+    }
+}
+
 #------------------------------------------------------------------------------
 # mentry::clamTheme
 #------------------------------------------------------------------------------
@@ -213,20 +421,25 @@ proc mentry::classicTheme {} {
 # mentry::clearlooksTheme
 #------------------------------------------------------------------------------
 proc mentry::clearlooksTheme {} {
-    variable themeDefaults
-    array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-readonlybackground	#efebe7 \
-	-foreground		black \
-	-foreground,background	black \
-	-disabledforeground	#b5b3ac \
-	-selectbackground	#7c99ad \
-	-selectforeground	#ffffff \
-	-selectborderwidth	0 \
-	-borderwidth		2 \
-	-labelpady		{2 2} \
-    ]
+    variable isAwTheme
+    if {$isAwTheme} {
+	awclearlooksTheme 
+    } else {
+	variable themeDefaults
+	array set themeDefaults [list \
+	    -background			white \
+	    -disabledbackground		"" \
+	    -readonlybackground		#efebe7 \
+	    -foreground			black \
+	    -foreground,background	black \
+	    -disabledforeground		#b5b3ac \
+	    -selectbackground		#7c99ad \
+	    -selectforeground		#ffffff \
+	    -selectborderwidth		0 \
+	    -borderwidth		2 \
+	    -labelpady			{2 2} \
+	]
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -299,7 +512,7 @@ proc mentry::krocTheme {} {
 	-disabledforeground	#b2b2b2 \
 	-selectbackground	#000000 \
 	-selectforeground	#ffffff \
-	-selectborderwidth	0 \
+	-selectborderwidth	1 \
 	-borderwidth		2 \
 	-labelpady		{2 2} \
     ]
@@ -449,19 +662,24 @@ proc mentry::winnativeTheme {} {
 # mentry::winxpblueTheme
 #------------------------------------------------------------------------------
 proc mentry::winxpblueTheme {} {
-    variable themeDefaults
-    array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
-	-disabledforeground	#565248 \
-	-selectbackground	#4a6984 \
-	-selectforeground	#ffffff \
-	-selectborderwidth	0 \
-	-borderwidth		2 \
-	-labelpady		{2 2} \
-    ]
+    variable isAwTheme
+    if {$isAwTheme} {
+	awwinxpblueTheme 
+    } else {
+	variable themeDefaults
+	array set themeDefaults [list \
+	    -background			white \
+	    -disabledbackground		"" \
+	    -foreground			black \
+	    -foreground,background	black \
+	    -disabledforeground		#565248 \
+	    -selectbackground		#4a6984 \
+	    -selectforeground		#ffffff \
+	    -selectborderwidth		0 \
+	    -borderwidth		2 \
+	    -labelpady			{2 2} \
+	]
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -478,25 +696,26 @@ proc mentry::xpnativeTheme {} {
 	-selectbackground	SystemHighlight \
 	-selectforeground	SystemHighlightText \
 	-selectborderwidth	0 \
+	-borderwidth		2 \
+	-labelpady		{2 4} \
     ]
+}
 
-    switch [winfo rgb . SystemButtonFace] {
-	"60652 59881 55512" -
-	"57568 57311 58339" -
-	"61680 61680 61680" {
-	    array set themeDefaults [list \
-		-borderwidth	2 \
-		-labelpady	{2 4} \
-	    ]
-	}
-
-	default {
-	    array set themeDefaults [list \
-		-borderwidth	2 \
-		-labelpady	{2 2} \
-	    ]
-	}
-    }
+#------------------------------------------------------------------------------
+# mentry::awTheme
+#------------------------------------------------------------------------------
+proc mentry::awTheme {} {
+    variable themeDefaults
+    array set themeDefaults [list \
+	-background		[styleConfig . -fieldbackground] \
+	-disabledbackground	[lindex [style map TEntry -fieldbackground] 1] \
+	-foreground		[styleConfig TEntry -foreground] \
+	-foreground,background	[styleConfig TEntry -foreground] \
+	-disabledforeground	[lindex [style map TEntry -foreground] 1] \
+	-selectbackground	[styleConfig . -selectbackground] \
+	-selectforeground	[styleConfig . -selectforeground] \
+	-selectborderwidth	[styleConfig . -selectborderwidth] \
+    ]
 }
 
 #
