@@ -7,7 +7,7 @@
 #   - Private procedures implementing the interactive cell editing
 #   - Private procedures used in bindings related to interactive cell editing
 #
-# Copyright (c) 2003-2018  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2003-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -296,7 +296,7 @@ proc tablelist::addBWidgetSpinBox {{name SpinBox}} {
 
     variable editWin
     array set editWin [list \
-	$name-creationCmd	"SpinBox %W -editable 1 -width 0" \
+	$name-creationCmd	"createBWidgetSpinBox %W" \
 	$name-putValueCmd	"%W configure -text %T" \
 	$name-getValueCmd	"%W cget -text" \
 	$name-putTextCmd	"%W configure -text %T" \
@@ -447,6 +447,14 @@ proc tablelist::addIncrDateTimeWidget {widgetType args} {
 	$name-isEntryLike	1 \
 	$name-reservedKeys	{Left Right Up Down} \
     ]
+    switch $widgetType {
+	dateentry {
+	    set editWin($name-creationCmd) "createIncrDateentry %W"
+	}
+	timeentry {
+	    set editWin($name-creationCmd) "createIncrTimeentry %W"
+	}
+    }
     if {$useClicks} {
 	lappend editWin($name-getValueCmd) -clicks
 	set editWin($name-useFormat) 0
@@ -1128,6 +1136,7 @@ proc tablelist::createMenubutton {w args} {
     if {[string compare $winSys "x11"] == 0} {
 	$menu configure -background $data(-background) \
 			-foreground $data(-foreground) \
+			-selectcolor $data(-foreground) \
 			-activebackground $data(-selectbackground) \
 			-activeforeground $data(-selectforeground) \
 			-activeborderwidth $data(-selectborderwidth)
@@ -1197,9 +1206,10 @@ proc tablelist::createTileEntry {w args} {
     # -borderwidth 2 setting, set the padding to another value.
     #
     set win [getTablelistPath $w]
-    switch -- [getCurrentTheme] {
+    variable currentTheme
+    switch -- $currentTheme {
 	aqua {
-	    set padding {0 0 0 -1}
+	    set padding 0
 	}
 
 	tileqt {
@@ -1243,14 +1253,15 @@ proc tablelist::createTileSpinbox {w args} {
     createTileAliases 
 
     #
-    # The style of the tile entry widget should have -borderwidth
+    # The style of the tile spinbox widget should have -borderwidth
     # 2 and -padding 1.  For those themes that don't honor the
     # -borderwidth 2 setting, set the padding to another value.
     #
     set win [getTablelistPath $w]
-    switch -- [getCurrentTheme] {
+    variable currentTheme
+    switch -- $currentTheme {
 	aqua {
-	    set padding {0 0 0 -1}
+	    set padding 0
 	}
 
 	tileqt {
@@ -1301,8 +1312,9 @@ proc tablelist::createTileCombobox {w args} {
     createTileAliases 
 
     set win [getTablelistPath $w]
-    if {[string compare [getCurrentTheme] "aqua"] == 0} {
-	styleConfig Tablelist.TCombobox -borderwidth 2 -padding {0 0 0 -1}
+    variable currentTheme
+    if {[string compare $currentTheme "aqua"] == 0} {
+	styleConfig Tablelist.TCombobox -borderwidth 2 -padding 0
     } else {
 	styleConfig Tablelist.TCombobox -borderwidth 2 -padding 1
     }
@@ -1312,6 +1324,12 @@ proc tablelist::createTileCombobox {w args} {
     foreach {opt val} $args {
 	$w configure $opt $val
     }
+
+    foreach event {<Map> <Unmap>} {
+	bind [ttk::combobox::PopdownWindow $w] $event {
+	    tablelist::invokeMotionHandler [tablelist::getTablelistPath %W]
+	}
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -1371,9 +1389,15 @@ proc tablelist::createTileMenubutton {w args} {
 
     set menu $w.menu
     menu $menu -tearoff 0 -postcommand [list tablelist::postMenuCmd $w]
+    foreach event {<Map> <Unmap>} {
+	bind $menu $event {
+	    tablelist::invokeMotionHandler [tablelist::getTablelistPath %W]
+	}
+    }
     if {[string compare $winSys "x11"] == 0} {
 	$menu configure -background $data(-background) \
 			-foreground $data(-foreground) \
+			-selectcolor $data(-foreground) \
 			-activebackground $data(-selectbackground) \
 			-activeforeground $data(-selectforeground) \
 			-activeborderwidth $data(-selectborderwidth)
@@ -1382,6 +1406,19 @@ proc tablelist::createTileMenubutton {w args} {
     $w configure -menu $menu
 }
 
+#------------------------------------------------------------------------------
+# tablelist::createBWidgetSpinBox
+#
+# Creates a BWidget SpinBox widget of the given path name for interactive cell
+# editing in a tablelist widget.
+#------------------------------------------------------------------------------
+proc tablelist::createBWidgetSpinBox {w args} {
+    eval [list SpinBox $w -editable 1 -width 0] $args
+
+    variable scalingpct
+    scaleutil::scaleBWidgetSpinBox $w $scalingpct
+}
+
 #------------------------------------------------------------------------------
 # tablelist::createBWidgetComboBox
 #
@@ -1390,7 +1427,9 @@ proc tablelist::createTileMenubutton {w args} {
 #------------------------------------------------------------------------------
 proc tablelist::createBWidgetComboBox {w args} {
     eval [list ComboBox $w -editable 1 -width 0] $args
-    ComboBox::_create_popup $w
+
+    variable scalingpct
+    scaleutil::scaleBWidgetComboBox $w $scalingpct
 
     foreach event {<Map> <Unmap>} {
 	bind $w.shell.listb $event {
@@ -1399,6 +1438,32 @@ proc tablelist::createBWidgetComboBox {w args} {
     }
 }
 
+#------------------------------------------------------------------------------
+# tablelist::createIncrDateentry
+#
+# Creates an [incr Widgets] dateentry of the given path name for interactive
+# cell editing in a tablelist widget.
+#------------------------------------------------------------------------------
+proc tablelist::createIncrDateentry {w args} {
+    eval [list iwidgets::dateentry $w] $args
+
+    variable scalingpct
+    scaleutil::scaleIncrDateentry $w $scalingpct
+}
+
+#------------------------------------------------------------------------------
+# tablelist::createIncrTimeentry
+#
+# Creates an [incr Widgets] timeentry of the given path name for interactive
+# cell editing in a tablelist widget.
+#------------------------------------------------------------------------------
+proc tablelist::createIncrTimeentry {w args} {
+    eval [list iwidgets::timeentry $w] $args
+
+    variable scalingpct
+    scaleutil::scaleIncrTimeentry $w $scalingpct
+}
+
 #------------------------------------------------------------------------------
 # tablelist::createIncrCombobox
 #
@@ -1406,7 +1471,11 @@ proc tablelist::createBWidgetComboBox {w args} {
 # cell editing in a tablelist widget.
 #------------------------------------------------------------------------------
 proc tablelist::createIncrCombobox {w args} {
-    eval [list iwidgets::combobox $w -dropdown 1 -editable 1 -width 0] $args
+    eval [list iwidgets::combobox $w -dropdown 1 -editable 1 -grab global \
+	  -width 0] $args
+
+    variable scalingpct
+    scaleutil::scaleIncrCombobox $w $scalingpct
 
     foreach event {<Map> <Unmap>} {
 	bind [$w component list] $event {+
@@ -1428,7 +1497,11 @@ proc tablelist::createIncrCombobox {w args} {
 # editing in a tablelist widget.
 #------------------------------------------------------------------------------
 proc tablelist::createOakleyCombobox {w args} {
-    eval [list combobox::combobox $w -editable 1 -width 0] $args
+    eval [list combobox::combobox $w -editable 1 -elementborderwidth 1 \
+	  -width 0] $args
+
+    variable scalingpct
+    scaleutil::scaleOakleyComboboxArrow $scalingpct
 
     foreach event {<Map> <Unmap>} {
 	bind $w.top.list $event {
@@ -1581,9 +1654,10 @@ proc tablelist::doEditCell {win row col restore {cmd ""} {charPos -1}} {
     $b mark set editMark $editIdx
 
     #
-    # Insert the binding tags $data(editwinTag) and TablelistEdit
-    # into the list of binding tags of some components
-    # of w, just before the respective path names
+    # Insert the binding tags $data(editwinTag) and TablelistEdit into the list
+    # of binding tags of some components of w, just before the respective path
+    # names.  In addition, insert the binding tag TablelistEditBreak into the
+    # same lists of binding tags, just after the respective widget class names
     #
     if {$isMentry} {
 	set compList [$w entries]
@@ -1596,6 +1670,10 @@ proc tablelist::doEditCell {win row col restore {cmd ""} {charPos -1}} {
 	set bindTags [bindtags $comp]
 	set idx [lsearch -exact $bindTags $comp]
 	bindtags $comp [linsert $bindTags $idx $data(editwinTag) TablelistEdit]
+
+	set bindTags [bindtags $comp]
+	set idx [lsearch -exact $bindTags [winfo class $comp]]
+	bindtags $comp [linsert $bindTags [incr idx] TablelistEditBreak]
     }
 
     #
@@ -1888,6 +1966,7 @@ proc tablelist::doCancelEditing win {
     set userData [list $row $col]
     genVirtualEvent $win <<TablelistCellRestored>> $userData
 
+    seeRow $win $row
     updateViewWhenIdle $win
     return ""
 }
@@ -1940,7 +2019,6 @@ proc tablelist::doFinishEditing win {
     #
     if {$data(rejected)} {
 	if {[winfo exists $data(bodyFrm)]} {
-	    seeCell $win $row $col
 	    if {[string compare [winfo class $w] "Mentry"] != 0} {
 		focus $data(editFocus)
 	    }
@@ -1972,6 +2050,7 @@ proc tablelist::doFinishEditing win {
 	genVirtualEvent $win <<TablelistCellUpdated>> $userData
     }
 
+    seeRow $win $row
     updateViewWhenIdle $win
     return $result
 }
@@ -2016,6 +2095,10 @@ proc tablelist::adjustTextHeight {w args} {
     if {[winfo exists $path]} {
 	[winfo parent $path] configure -height [winfo reqheight $path]
     }
+
+    set win [getTablelistPath $path]
+    upvar ::tablelist::ns${win}::data data
+    seeRow $win $data(editRow)
 }
 
 #------------------------------------------------------------------------------
@@ -2128,6 +2211,30 @@ proc tablelist::adjustEditWindow {win pixels} {
 	setImgLabelWidth $data(body) editAuxMark $auxWidth
     }
 
+    switch [winfo class $data(bodyFrmEd)] {
+	Text -
+	Ctext { set amount 4 }
+	TEntry {
+	    variable currentTheme
+	    if {[string compare $currentTheme "aqua"] == 0} {
+		variable newAquaSupport
+		set amount [expr {$newAquaSupport ? 7 : 5}]
+	    } else {
+		set amount 3
+	    }
+	}
+	Mentry {
+	    variable currentTheme
+	    if {$::mentry::usingTile &&
+		[string compare $currentTheme "aqua"] == 0} {
+		set amount 5
+	    } else {
+		set amount 3
+	    }
+	}
+	default { set amount 3 }
+    }
+
     #
     # Compute an appropriate width and horizontal
     # padding for the frame containing the edit window
@@ -2138,23 +2245,13 @@ proc tablelist::adjustEditWindow {win pixels} {
 	[set reqWidth [winfo reqwidth $data(bodyFrmEd)]] <=
 	$pixels + 2*$data(charWidth)} {
 	set width $reqWidth
-	set padX [expr {$reqWidth <= $pixels ? -3 : ($pixels - $reqWidth) / 2}]
+	set padX [expr {$reqWidth <= $pixels ?
+			-$amount : ($pixels - $reqWidth) / 2}]
     } else {
 	if {$editWin($name-usePadX)} {
 	    set amount $data(charWidth)
-	} else {
-	    switch -- $name {
-		text { set amount 4 }
-		ttk::entry {
-		    if {[string compare [getCurrentTheme] "aqua"] == 0} {
-			set amount 5
-		    } else {
-			set amount 3
-		    }
-		}
-		default { set amount 3 }
-	    }
 	}
+
 	set width [expr {$pixels + 2*$amount}]
 	set padX -$amount
     }
@@ -2252,6 +2349,7 @@ proc tablelist::saveEditData win {
 proc tablelist::saveEditConfigOpts w {
     regexp {^(.+)\.body\.f\.(e.*)$} $w dummy win tail
     upvar ::tablelist::ns${win}::data data
+    set isMentry [expr {[string compare [winfo class $w] "Mentry"] == 0}]
 
     foreach configSet [$w configure] {
 	if {[llength $configSet] != 2} {
@@ -2259,7 +2357,10 @@ proc tablelist::saveEditConfigOpts w {
 	    set current [lindex $configSet 4]
 	    if {[string compare $default $current] != 0} {
 		set opt [lindex $configSet 0]
-		set data($tail$opt) $current
+		if {[string compare $opt "-class"] != 0 &&
+		    !(!$isMentry && [string compare $opt "-body"] == 0)} {
+		    set data($tail$opt) $current
+		}
 	    }
 	}
     }
@@ -2384,13 +2485,10 @@ proc tablelist::restoreEditData win {
 proc tablelist::restoreEditConfigOpts w {
     regexp {^(.+)\.body\.f\.(e.*)$} $w dummy win tail
     upvar ::tablelist::ns${win}::data data
-    set isMentry [expr {[string compare [winfo class $w] "Mentry"] == 0}]
 
     foreach name [array names data $tail-*] {
 	set opt [string range $name [string last "-" $name] end]
-	if {!$isMentry || [string compare $opt "-body"] != 0} {
-	    $w configure $opt $data($name)
-	}
+	$w configure $opt $data($name)
 	unset data($name)
     }
 
@@ -2461,8 +2559,6 @@ proc tablelist::defineTablelistEdit {} {
 	    set tablelist::priv(y) ""
 	    after cancel $tablelist::priv(afterId)
 	    set tablelist::priv(afterId) ""
-	    set tablelist::priv(justReleased) 1
-	    after 100 [list set tablelist::priv(justReleased) 0]
 	    set tablelist::priv(releasedInEditWin) 1
 	    if {!$tablelist::priv(clickedInEditWin)} {
 		if {$tablelist::priv(justClicked)} {
@@ -2526,15 +2622,15 @@ proc tablelist::defineTablelistEdit {} {
 	    tablelist::goToNextPrevCell %W -1 0 0
 	}
     }
-    foreach direction {Left Right} amount {-1 1} {
-	bind TablelistEdit <$direction> [format {
+    foreach dir {Left Right} amount {-1 1} {
+	bind TablelistEdit <$dir> [format {
 	    if {![tablelist::isKeyReserved %%W %%K]} {
 		tablelist::goLeftRight %%W %d
 	    }
 	} $amount]
     }
-    foreach direction {Up Down} amount {-1 1} {
-	bind TablelistEdit <$direction> [format {
+    foreach dir {Up Down} amount {-1 1} {
+	bind TablelistEdit <$dir> [format {
 	    if {![tablelist::isKeyReserved %%W %%K]} {
 		tablelist::goUpDown %%W %d
 	    }
@@ -2603,32 +2699,151 @@ proc tablelist::defineTablelistEdit {} {
 
     #
     # Define some bindings for the binding tag TablelistEdit that
-    # propagate the mousewheel events to the tablelist's body
+    # redirect the mouse wheel events to the containing scrollable
+    # frame (if any) or propagate them to the tablelist's body
     #
     catch {
 	bind TablelistEdit <MouseWheel> {
-	    if {![tablelist::hasMouseWheelBindings %W] &&
-		![tablelist::isComboTopMapped %W]} {
-		tablelist::genMouseWheelEvent \
-		    [[tablelist::getTablelistPath %W] bodypath] %D
+	    if {[tablelist::hasMouseWheelBindings %W y]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		set tablelist::w [$tablelist::W cget -ymousewheelwindow]
+		if {[winfo exists $tablelist::w] &&
+		    ![mwutil::hasFocus $tablelist::W]} {
+		    mwutil::genMouseWheelEvent $tablelist::w \
+			<MouseWheel> %X %Y %D
+		    break
+		}
+	    } elseif {![tablelist::isComboTopMapped %W] &&
+		      ![tablelist::isMenuPosted %W]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		mwutil::genMouseWheelEvent [$tablelist::W bodypath] \
+		    <MouseWheel> %X %Y %D
 	    }
 	}
+	bind TablelistEditBreak <MouseWheel> { break }
+
 	bind TablelistEdit <Option-MouseWheel> {
-	    if {![tablelist::hasMouseWheelBindings %W] &&
-		![tablelist::isComboTopMapped %W]} {
-		tablelist::genOptionMouseWheelEvent \
-		    [[tablelist::getTablelistPath %W] bodypath] %D
+	    if {[tablelist::hasMouseWheelBindings %W y]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		set tablelist::w [$tablelist::W cget -ymousewheelwindow]
+		if {[winfo exists $tablelist::w] &&
+		    ![mwutil::hasFocus $tablelist::W]} {
+		    mwutil::genMouseWheelEvent $tablelist::w \
+			<Option-MouseWheel> %X %Y %D
+		    break
+		}
+	    } elseif {![tablelist::isComboTopMapped %W] &&
+		      ![tablelist::isMenuPosted %W]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		mwutil::genMouseWheelEvent [$tablelist::W bodypath] \
+		    <Option-MouseWheel> %X %Y %D
 	    }
 	}
+	bind TablelistEditBreak <Option-MouseWheel> { break }
     }
-    foreach detail {4 5} {
-	bind TablelistEdit <Button-$detail> [format {
-	    if {![tablelist::hasMouseWheelBindings %%W] &&
-		![tablelist::isComboTopMapped %%W]} {
-		event generate \
-		    [[tablelist::getTablelistPath %%W] bodypath] <Button-%s>
+    catch {
+	bind TablelistEdit <Shift-MouseWheel> {
+	    if {[tablelist::hasMouseWheelBindings %W x]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		set tablelist::w [$tablelist::W cget -xmousewheelwindow]
+		if {[winfo exists $tablelist::w] &&
+		    ![mwutil::hasFocus $tablelist::W]} {
+		    mwutil::genMouseWheelEvent $tablelist::w \
+			<Shift-MouseWheel> %X %Y %D
+		    break
+		}
+	    } elseif {![tablelist::isComboTopMapped %W] &&
+		      ![tablelist::isMenuPosted %W]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		mwutil::genMouseWheelEvent [$tablelist::W bodypath] \
+		    <Shift-MouseWheel> %X %Y %D
 	    }
-	} $detail]
+	}
+	bind TablelistEditBreak <Shift-MouseWheel> { break }
+
+	bind TablelistEdit <Shift-Option-MouseWheel> {
+	    if {[tablelist::hasMouseWheelBindings %W x]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		set tablelist::w [$tablelist::W cget -xmousewheelwindow]
+		if {[winfo exists $tablelist::w] &&
+		    ![mwutil::hasFocus $tablelist::W]} {
+		    mwutil::genMouseWheelEvent $tablelist::w \
+			<Shift-Option-MouseWheel> %X %Y %D
+		    break
+		}
+	    } elseif {![tablelist::isComboTopMapped %W] &&
+		      ![tablelist::isMenuPosted %W]} {
+		set tablelist::W [tablelist::getTablelistPath %W]
+		mwutil::genMouseWheelEvent [$tablelist::W bodypath] \
+		    <Shift-Option-MouseWheel> %X %Y %D
+	    }
+	}
+	bind TablelistEditBreak <Shift-Option-MouseWheel> { break }
+    }
+    if {[string compare $winSys "x11"] == 0} {
+	foreach detail {4 5} {
+	    bind TablelistEdit <Button-$detail> [format {
+		if {[tablelist::hasMouseWheelBindings %%W y]} {
+		    set tablelist::W [tablelist::getTablelistPath %%W]
+		    set tablelist::w [$tablelist::W cget -ymousewheelwindow]
+		    if {[winfo exists $tablelist::w] &&
+			![mwutil::hasFocus $tablelist::W]} {
+			event generate $tablelist::w <Button-%s> \
+			    -rootx %%X -rooty %%Y
+			break
+		    }
+		} elseif {![tablelist::isComboTopMapped %%W] &&
+			  ![tablelist::isMenuPosted %%W]} {
+		    set tablelist::W [tablelist::getTablelistPath %%W]
+		    event generate [$tablelist::W bodypath] <Button-%s> \
+			-rootx %%X -rooty %%Y
+		}
+	    } $detail $detail]
+	    bind TablelistEditBreak <Button-$detail> { break }
+
+	    bind TablelistEdit <Shift-Button-$detail> [format {
+		if {[tablelist::hasMouseWheelBindings %%W x]} {
+		    set tablelist::W [tablelist::getTablelistPath %%W]
+		    set tablelist::w [$tablelist::W cget -xmousewheelwindow]
+		    if {[winfo exists $tablelist::w] &&
+			![mwutil::hasFocus $tablelist::W]} {
+			event generate $tablelist::w <Shift-Button-%s> \
+			    -rootx %%X -rooty %%Y
+			break
+		    }
+		} elseif {![tablelist::isComboTopMapped %%W] &&
+			  ![tablelist::isMenuPosted %%W]} {
+		    set tablelist::W [tablelist::getTablelistPath %%W]
+		    event generate [$tablelist::W bodypath] <Shift-Button-%s> \
+			-rootx %%X -rooty %%Y
+		}
+	    } $detail $detail]
+	    bind TablelistEditBreak <Shift-Button-$detail> { break }
+	}
+
+	if {$::tk_version >= 8.7 &&
+	    [package vcompare $::tk_patchLevel "8.7a3"] >= 0} {
+	    foreach detail {6 7} {
+		bind TablelistEdit <Button-$detail> [format {
+		    if {[tablelist::hasMouseWheelBindings %%W x]} {
+			set tablelist::W [tablelist::getTablelistPath %%W]
+			set tablelist::w [$tablelist::W cget -xmousewheelwindow]
+			if {[winfo exists $tablelist::w] &&
+			    ![mwutil::hasFocus $tablelist::W]} {
+			    event generate $tablelist::w <Button-%s> \
+				-rootx %%X -rooty %%Y
+			    break
+			}
+		    } elseif {![tablelist::isComboTopMapped %%W] &&
+			      ![tablelist::isMenuPosted %%W]} {
+			set tablelist::W [tablelist::getTablelistPath %%W]
+			event generate [$tablelist::W bodypath] <Button-%s> \
+			    -rootx %%X -rooty %%Y
+		    }
+		} $detail $detail]
+		bind TablelistEditBreak <Button-$detail> { break }
+	    }
+	}
     }
 }
 
@@ -2650,12 +2865,12 @@ proc tablelist::insertChar {w str} {
 	return -code break ""
     } elseif {[regexp {^(T?Entry|TCombobox|T?Spinbox)$} $class]} {
 	if {[string match "T*" $class]} {
-	    if {[string length [info procs "::ttk::entry::Insert"]] != 0} {
+	    if {[llength [info procs "::ttk::entry::Insert"]] != 0} {
 		ttk::entry::Insert $w $str
 	    } else {
 		tile::entry::Insert $w $str
 	    }
-	} elseif {[string length [info procs "::tk::EntryInsert"]] != 0} {
+	} elseif {[llength [info procs "::tk::EntryInsert"]] != 0} {
 	    tk::EntryInsert $w $str
 	} else {
 	    tkEntryInsert $w $str
@@ -2901,30 +3116,6 @@ proc tablelist::goToPriorNextPage {w amount} {
     return -code break ""
 }
 
-#------------------------------------------------------------------------------
-# tablelist::genMouseWheelEvent
-#
-# Generates a <MouseWheel> event with the given delta on the widget w.
-#------------------------------------------------------------------------------
-proc tablelist::genMouseWheelEvent {w delta} {
-    set focus [focus -displayof $w]
-    focus $w
-    event generate $w <MouseWheel> -delta $delta
-    focus $focus
-}
-
-#------------------------------------------------------------------------------
-# tablelist::genOptionMouseWheelEvent
-#
-# Generates an <Option-MouseWheel> event with the given delta on the widget w.
-#------------------------------------------------------------------------------
-proc tablelist::genOptionMouseWheelEvent {w delta} {
-    set focus [focus -displayof $w]
-    focus $w
-    event generate $w <Option-MouseWheel> -delta $delta
-    focus $focus
-}
-
 #------------------------------------------------------------------------------
 # tablelist::isKeyReserved
 #
@@ -2945,18 +3136,22 @@ proc tablelist::isKeyReserved {w keySym} {
 # tablelist::hasMouseWheelBindings
 #
 # Checks whether the given widget, which is assumed to be the edit window or
-# one of its descendants, has mouse wheel bindings.
+# one of its descendants, has mouse wheel bindings for the given axis (x or y).
 #------------------------------------------------------------------------------
-proc tablelist::hasMouseWheelBindings w {
-    if {[regexp {^(Text|Ctext|TCombobox|TSpinbox)$} [winfo class $w]]} {
-	return 1
+proc tablelist::hasMouseWheelBindings {w axis} {
+    if {[string compare $axis "x"] == 0} {
+	return [regexp {^(Text|Ctext)$} [winfo class $w]]
     } else {
-	set bindTags [bindtags $w]
-	return [expr {([lsearch -exact $bindTags "MentryDateTime"] >= 0 ||
-		       [lsearch -exact $bindTags "MentryMeridian"] >= 0 ||
-		       [lsearch -exact $bindTags "MentryIPAddr"] >= 0 ||
-		       [lsearch -exact $bindTags "MentryIPv6Addr"] >= 0) &&
-		      ($mentry::version >= 3.2)}]
+	if {[regexp {^(Text|Ctext|TCombobox|TSpinbox)$} [winfo class $w]]} {
+	    return 1
+	} else {
+	    set bindTags [bindtags $w]
+	    return [expr {([lsearch -exact $bindTags "MentryDateTime"] >= 0 ||
+			   [lsearch -exact $bindTags "MentryMeridian"] >= 0 ||
+			   [lsearch -exact $bindTags "MentryIPAddr"] >= 0 ||
+			   [lsearch -exact $bindTags "MentryIPv6Addr"] >= 0) &&
+			  ($mentry::version >= 3.2)}]
+	}
     }
 }
 
@@ -2978,3 +3173,20 @@ proc tablelist::isComboTopMapped w {
 	return 0
     }
 }
+
+#------------------------------------------------------------------------------
+# tablelist::isMenuPosted
+#
+# Checks whether the given widget is a menubutton having its menu posted.  This
+# is needed in our binding scripts for mouse wheel events to make sure that
+# they won't generate an endless loop, because of the global grab set on the
+# menu when the latter is posted.
+#------------------------------------------------------------------------------
+proc tablelist::isMenuPosted w {
+    if {[string compare [winfo class $w] "Menubutton"] == 0 &&
+	[winfo ismapped [$w cget -menu]]} {
+	return 1
+    } else {
+	return 0
+    }
+}
