@@ -5,10 +5,9 @@
 # Structure of the module:
 #   - Public procedures related to tile themes
 #   - Private procedures related to tile themes
-#   - Private procedures performing RGB <-> HSV conversions
 #   - Private procedures related to global KDE configuration options
 #
-# Copyright (c) 2005-2019  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2005-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -16,6 +15,15 @@
 # ========================================
 #
 
+#------------------------------------------------------------------------------
+# tablelist::getCurrentTheme
+#
+# Returns the current tile theme.
+#------------------------------------------------------------------------------
+proc tablelist::getCurrentTheme {} {
+    return [mwutil::currentTheme]
+}
+
 #------------------------------------------------------------------------------
 # tablelist::setThemeDefaults
 #
@@ -23,14 +31,64 @@
 # tablelist configuration options and updates the array configSpecs.
 #------------------------------------------------------------------------------
 proc tablelist::setThemeDefaults {} {
+    set currentTheme [mwutil::currentTheme]
+    variable isAwTheme \
+	[llength [info commands ::ttk::theme::${currentTheme}::setTextColors]]
+
     variable themeDefaults
-    if {[catch {[getCurrentTheme]Theme}] != 0} {
+    if {[catch {${currentTheme}Theme}] != 0} {
 	#
 	# Fall back to the "default" theme (which is the root of all
 	# themes) and then override the options set by the current one
 	#
 	defaultTheme 
-	array set themeDefaults [style configure .]
+	array set themeDefaults [styleConfig .]
+
+	if {[set bg [styleConfig . -background]] ne ""} {
+	    array set themeDefaults [list \
+		-labelbackground	$bg \
+		-labeldeactivatedBg	$bg \
+		-labeldisabledBg	$bg \
+		-labelactiveBg		$bg \
+		-labelpressedBg		$bg \
+	    ]
+	}
+
+	if {[set fg [styleConfig . -foreground]] ne ""} {
+	    array set themeDefaults [list \
+		-labelforeground	$fg \
+		-labelactiveFg		$fg \
+		-labelpressedFg		$fg \
+		-arrowcolor		$fg \
+	    ]
+	}
+
+	array set arr [style map . -background]
+	if {[info exists arr(active)]} {
+	    set activeBg $arr(active)
+	    array set themeDefaults [list \
+		-labelactiveBg		$activeBg \
+		-labelpressedBg		$activeBg \
+	    ]
+	} elseif {[set highlightBg [styleConfig . -highlightcolor]] ne ""} {
+	    array set themeDefaults [list \
+		-labelactiveBg		$highlightBg \
+		-labelpressedBg		$highlightBg \
+	    ]
+	}
+	if {[info exists arr(pressed)]} {
+	    set themeDefaults(-labelpressedBg) $arr(pressed)
+	}
+
+	unset arr
+	array set arr [style map . -foreground]
+	if {[info exists arr(disabled)]} {
+	    set disabledFg $arr(disabled)
+	    array set themeDefaults [list \
+		-disabledforeground	$disabledFg \
+		-labeldisabledFg	$disabledFg \
+	    ]
+	}
     }
 
     if {[string length $themeDefaults(-arrowcolor)] == 0} {
@@ -84,8 +142,8 @@ proc tablelist::altTheme {} {
 	-labelborderwidth	2 \
 	-labelpady		1 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
-	-treestyle		winnative \
+	-arrowstyle		[defaultX11ArrowStyle] \
+	-treestyle		gtk \
     ]
 }
 
@@ -93,115 +151,199 @@ proc tablelist::altTheme {} {
 # tablelist::aquaTheme
 #------------------------------------------------------------------------------
 proc tablelist::aquaTheme {} {
+    variable newAquaSupport
+    if {$newAquaSupport} {
+	set darkMode   [tk::unsupported::MacWindowStyle isdark .]
+	set disabledFg [expr {$darkMode ? "#646464" : "#b1b1b1"}]
+    } else {
+	set disabledFg #b1b1b1
+    }
+
     scan $::tcl_platform(osVersion) "%d" majorOSVersion
-    if {$majorOSVersion >= 14} {			;# OS X 10.10 or higher
-	set labelBg		#f4f4f4
-	set labeldeactivatedBg	#ffffff
-	set labeldisabledBg	#ffffff
-	set labelpressedBg	#e4e4e4
-	set arrowColor		#404040
-    } elseif {$majorOSVersion >= 11} {			;# OS X 10.7 or higher
+    if {$majorOSVersion >= 14} {			;# OS X 10.10 or later
+	if {$newAquaSupport} {
+	    if {$darkMode} {
+		set labelBg		#323232
+		set labeldeactivatedBg	#323232
+		set labeldisabledBg	""
+		set labelpressedBg	#323232
+		set arrowColor		#808080
+	    } else {
+		set labelBg		#eeeeee
+		set labeldeactivatedBg	#f6f6f6
+		set labeldisabledBg	""
+		set labelpressedBg	#eeeeee
+		set arrowColor		#404040
+	    }
+	} else {
+	    set labelBg			#f6f6f6
+	    set labeldeactivatedBg	#ffffff
+	    set labeldisabledBg		#ffffff
+	    set labelpressedBg		#e9e9e9
+	    set arrowColor		#404040
+	}
+    } elseif {$majorOSVersion >= 11} {			;# OS X 10.7 or later
+	set labelBg		#f3f3f3
+	set labeldeactivatedBg	#f3f3f3
+	set labeldisabledBg	#f3f3f3
+	set labelpressedBg	#d5d5d5
+	set arrowColor		#777777
+    } else {
 	set labelBg		#efefef
 	set labeldeactivatedBg	#efefef
 	set labeldisabledBg	#efefef
-	set labelpressedBg	#cbcbcb
+	set labelpressedBg	#dddddd
 	set arrowColor		#777777
-    } else {
-	set labelBg		#e9e8e8
-	set labeldeactivatedBg	#e9e8e8
-	set labeldisabledBg	#e9e8e8
-	set labelpressedBg	#d2d2d2
-	set arrowColor		#717171
     }
 
     switch [winfo rgb . systemMenuActive] {
 	"13621 29041 52685" -
 	"32256 44288 55552" {				;# Blue Cocoa/Carbon
-	    if {$majorOSVersion >= 14} {		;# OS X 10.10 or higher
-		set stripeBg			#f2f2f2
-		set labelselectedBg		#f4f4f4
-		set labelselectedpressedBg	#e4e4e4
-	    } elseif {$majorOSVersion >= 11} {		;# OS X 10.7 or higher
-		set stripeBg			#f0f4f9
-		set labelselectedBg		#80b8f0
-		set labelselectedpressedBg	#417ddc
+	    if {$majorOSVersion >= 14} {		;# OS X 10.10 or later
+		if {$newAquaSupport} {
+		    if {$darkMode} {
+			set stripeBg			#292929
+			set labelselectedBg		#323232
+			set labelselectedpressedBg	#323232
+		    } else {
+			set stripeBg			#f5f5f5
+			set labelselectedBg		#eeeeee
+			set labelselectedpressedBg	#eeeeee
+		    }
+		} else {
+		    set stripeBg		#f5f5f5
+		    set labelselectedBg		#f6f6f6
+		    set labelselectedpressedBg	#e9e9e9
+		}
+	    } elseif {$majorOSVersion >= 11} {		;# OS X 10.7 or later
+		set stripeBg			#f3f6fa
+		set labelselectedBg		#91c5f3
+		set labelselectedpressedBg	#5092e3
 	    } else {
 		set stripeBg			#edf3fe
-		set labelselectedBg		#7ab2e9
-		set labelselectedpressedBg	#679ed5
+		set labelselectedBg		#80b8ef
+		set labelselectedpressedBg	#69aaeb
 	    }
 	}
 
 	"24415 27499 31354" -
 	"39680 43776 48384" {				;# Graphite Cocoa/Carbon
-	    if {$majorOSVersion >= 14} {		;# OS X 10.10 or higher
-		set stripeBg			#f2f2f2
-		set labelselectedBg		#f4f4f4
-		set labelselectedpressedBg	#e4e4e4
-	    } elseif {$majorOSVersion >= 11} {		;# OS X 10.7 or higher
-		set stripeBg			#f4f5f7
-		set labelselectedBg		#9ba7b5
-		set labelselectedpressedBg	#636e88
+	    if {$majorOSVersion >= 14} {		;# OS X 10.10 or later
+		if {$newAquaSupport} {
+		    if {$darkMode} {
+			set stripeBg			#292929
+			set labelselectedBg		#323232
+			set labelselectedpressedBg	#323232
+		    } else {
+			set stripeBg			#f5f5f5
+			set labelselectedBg		#eeeeee
+			set labelselectedpressedBg	#eeeeee
+		    }
+		} else {
+		    set stripeBg		#f5f5f5
+		    set labelselectedBg		#f6f6f6
+		    set labelselectedpressedBg	#e9e9e9
+		}
+	    } elseif {$majorOSVersion >= 11} {		;# OS X 10.7 or later
+		set stripeBg			#f6f7f9
+		set labelselectedBg		#abb6c2
+		set labelselectedpressedBg	#76829a
 	    } else {
 		set stripeBg			#f0f0f0
-		set labelselectedBg		#b6c2cd
-		set labelselectedpressedBg	#a7b3be
+		set labelselectedBg		#c0c7ce
+		set labelselectedpressedBg	#afb7c0
 	    }
 	}
     }
 
     #
-    # Get an approximation of alternateSelectedControlColor
+    # Get the default value of the -selectbackground option
     #
-    switch [winfo rgb . systemHighlight] {
-	"65535 48058 47288"	{ set selectBg #fc2125 }
-	"65535 57311 46003"	{ set selectBg #fd8208 }
-	"65535 61423 45231"	{ set selectBg #fec309 }
-	"49343 63222 44460"	{ set selectBg #56d72b }
-	"45746 55246 65535"	{ set selectBg #0950d0 }
-	"63478 54484 65535"	{ set selectBg #bf57da }
-	"65535 49087 53969"	{ set selectBg #7b0055 }
-	"60909 57053 51914"	{ set selectBg #90714c }
-	"55512 55512 56539"	{ set selectBg #5c5c60 }
+    if {$majorOSVersion >= 18} {			;# OS X 10.14 or later
+	if {$newAquaSupport} {
+	    variable channel
+	    if {[info exists channel]} {	;# see proc condOpenPipeline
+		set rgb [gets $channel]
 
-	"51143 53456 56281"	{ set selectBg #738499 }
-	"50887 50887 50887"	{ set selectBg #7f7f7f }
-	"46516 54741 65535"	{ set selectBg #3875d7 }
-	"64506 60908 29556"	{ set selectBg #ffc11f }
-	"65535 45487 35978"	{ set selectBg #f34648 }
-	"65535 53968 33154"	{ set selectBg #ff8a22 }
-	"50114 63994 37263"	{ set selectBg #66c547 }
-	"59879 47290 65535"	{ set selectBg #8c4eb8 }
-
-	default {
-	    set rgb [winfo rgb . systemHighlight]
-	    foreach {h s v} [eval rgb2hsv $rgb] {}
-
-	    set s [expr {$s*4.0/3.0}]
-	    if {$s > 1.0} {
-		set s 1.0
+		puts $channel "exit"
+		flush $channel
+		close $channel
+		unset channel
+	    } else {
+		set rgb [winfo rgb . systemSelectedTextBackgroundColor]
 	    }
 
-	    set v [expr {$v*3.0/4.0}]
-	    if {$v > 1.0} {
-		set v 1.0
+	    if {$darkMode} {
+		switch $rgb {
+		    "16191 25443 35723"	{ set selectBg #0258d0 }
+		    "28784 22102 28784"	{ set selectBg #7f3280 }
+		    "34952 22102 28270"	{ set selectBg #c83179 }
+		    "35723 22359 22616"	{ set selectBg #d03439 }
+		    "34952 25957 18247"	{ set selectBg #c86003 }
+		    "35466 30069 19018"	{ set selectBg #cd8f0e }
+		    "23644 30326 21331"	{ set selectBg #42912a }
+		    "65535 65535 65535"	{ set selectBg #686868 }
+		    default	{ set selectBg systemHighlightAlternate }
+		}
+	    } else {
+		switch $rgb {
+		    "46003 55255 65535"	{ set selectBg #0363e1 }
+		    "57311 50629 57311"	{ set selectBg #7d2a7e }
+		    "64764 51914 58082"	{ set selectBg #d93b85 }
+		    "62965 50115 50629"	{ set selectBg #c3252b }
+		    "64764 55769 48059"	{ set selectBg #d96b0a }
+		    "65278 59881 48830"	{ set selectBg #de9e15 }
+		    "53456 60138 51143"	{ set selectBg #4da032 }
+		    "57568 57568 57568"	{ set selectBg #808080 }
+		    default	{ set selectBg systemHighlightAlternate }
+		}
+	    }
+	} else {
+	    switch [winfo rgb . systemHighlight] {
+		"45746 55246 65535"	{ set selectBg #0363e1 }
+		"63478 54484 65535"	{ set selectBg #7d2a7e }
+		"65535 49087 53969"	{ set selectBg #d93b85 }
+		"65535 48058 47288"	{ set selectBg #c3252b }
+		"65535 57311 46003"	{ set selectBg #d96b0a }
+		"65535 61423 45231"	{ set selectBg #de9e15 }
+		"49343 63222 44460"	{ set selectBg #4da032 }
+		"55512 55512 56539"	{ set selectBg #808080 }
+		default		{ set selectBg systemHighlightAlternate }
 	    }
-
-	    set rgb [hsv2rgb $h $s $v]
-	    set selectBg [eval format "#%04x%04x%04x" $rgb]
 	}
+    } else {
+	set selectBg systemHighlightAlternate
     }
 
     variable themeDefaults
+    if {$newAquaSupport} {
+	array set themeDefaults [list \
+	    -background			systemTextBackgroundColor \
+	    -foreground			systemTextColor \
+	    -labelforeground		systemLabelColor \
+	    -labelactiveFg		systemLabelColor \
+	    -labelpressedFg		systemLabelColor \
+	    -labelselectedFg		systemLabelColor \
+	    -labelselectedpressedFg	systemLabelColor \
+	]
+    } else {
+	array set themeDefaults [list \
+	    -background			systemWindowBody \
+	    -foreground			systemModelessDialogActiveText \
+	    -labelforeground		systemModelessDialogActiveText \
+	    -labelactiveFg		systemModelessDialogActiveText \
+	    -labelpressedFg		systemModelessDialogActiveText \
+	    -labelselectedFg		systemModelessDialogActiveText \
+	    -labelselectedpressedFg	systemModelessDialogActiveText \
+	]
+    }
     array set themeDefaults [list \
-	-background		white \
-	-foreground		black \
-	-disabledforeground	#a3a3a3 \
+	-disabledforeground	$disabledFg \
 	-stripebackground	$stripeBg \
 	-selectbackground	$selectBg \
 	-selectforeground	white \
 	-selectborderwidth	0 \
-	-font			{"Lucida Grande" 12} \
+	-font			TkTextFont \
 	-labelbackground	$labelBg \
 	-labeldeactivatedBg	$labeldeactivatedBg \
 	-labeldisabledBg	$labeldisabledBg \
@@ -209,13 +351,8 @@ proc tablelist::aquaTheme {} {
 	-labelpressedBg		$labelpressedBg \
 	-labelselectedBg	$labelselectedBg \
 	-labelselectedpressedBg	$labelselectedpressedBg \
-	-labelforeground	black \
-	-labeldisabledFg	#a3a3a3 \
-	-labelactiveFg		black \
-	-labelpressedFg		black \
-	-labelselectedFg	black \
-	-labelselectedpressedFg	black \
-	-labelfont		{"Lucida Grande" 11} \
+	-labeldisabledFg	$disabledFg \
+	-labelfont		TkHeadingFont \
 	-labelborderwidth	1 \
 	-labelpady		1 \
 	-arrowcolor		$arrowColor \
@@ -223,7 +360,7 @@ proc tablelist::aquaTheme {} {
     ]
 
     variable pngSupported
-    if {$majorOSVersion >= 14} {			;# OS X 10.10 or higher
+    if {$majorOSVersion >= 14} {			;# OS X 10.10 or later
 	set themeDefaults(-arrowstyle) flatAngle7x4
     } elseif {$pngSupported} {
 	set themeDefaults(-arrowstyle) photo7x7
@@ -258,7 +395,7 @@ proc tablelist::AquativoTheme {} {
 	-labelfont		TkDefaultFont \
 	-labelborderwidth	2 \
 	-labelpady		1 \
-	-arrowcolor		#717171 \
+	-arrowcolor		#777777 \
 	-arrowstyle		flat7x7 \
 	-treestyle		aqua \
     ]
@@ -301,6 +438,7 @@ proc tablelist::aquativoTheme {} {
 #------------------------------------------------------------------------------
 proc tablelist::ArcTheme {} {
     variable themeDefaults
+    variable scalingpct
     array set themeDefaults [list \
 	-background		white \
 	-foreground		#5c616c \
@@ -324,10 +462,125 @@ proc tablelist::ArcTheme {} {
 	-labelpady		0 \
 	-arrowcolor		#5c616c \
 	-arrowstyle		flatAngle10x6 \
-	-treestyle		arc \
+	-treestyle		classic$scalingpct \
     ]
 }
 
+#------------------------------------------------------------------------------
+# tablelist::arcTheme
+#------------------------------------------------------------------------------
+proc tablelist::arcTheme {} {
+    variable isAwTheme
+    if {$isAwTheme} {
+	awarcTheme 
+    } else {
+	return -code error			;# handled by setThemeDefaults
+    }
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awarcTheme
+#------------------------------------------------------------------------------
+proc tablelist::awarcTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#e6e7e8 \
+	-treestyle		classic$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awblackTheme
+#------------------------------------------------------------------------------
+proc tablelist::awblackTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#333333 \
+	-treestyle		bicolor$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awbreezeTheme
+#------------------------------------------------------------------------------
+proc tablelist::awbreezeTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#e0e1e2 \
+	-treestyle		bicolor$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awclearlooksTheme
+#------------------------------------------------------------------------------
+proc tablelist::awclearlooksTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#e0dcd8 \
+	-treestyle		plain$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awdarkTheme
+#------------------------------------------------------------------------------
+proc tablelist::awdarkTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#242a2c \
+	-treestyle		white$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awlightTheme
+#------------------------------------------------------------------------------
+proc tablelist::awlightTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#d9d9d8 \
+	-treestyle		bicolor$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::awwinxpblueTheme
+#------------------------------------------------------------------------------
+proc tablelist::awwinxpblueTheme {} {
+    awTheme 
+    variable themeDefaults
+    variable scalingpct
+    array set themeDefaults [list \
+	-stripebackground	#dddac9 \
+	-treestyle		bicolor$scalingpct \
+    ]
+}
+
+#------------------------------------------------------------------------------
+# tablelist::blackTheme
+#------------------------------------------------------------------------------
+proc tablelist::blackTheme {} {
+    variable isAwTheme
+    if {$isAwTheme} {
+	awblackTheme 
+    } else {
+	return -code error			;# handled by setThemeDefaults
+    }
+}
+
 #------------------------------------------------------------------------------
 # tablelist::blueTheme
 #------------------------------------------------------------------------------
@@ -360,6 +613,18 @@ proc tablelist::blueTheme {} {
     ]
 }
 
+#------------------------------------------------------------------------------
+# tablelist::breezeTheme
+#------------------------------------------------------------------------------
+proc tablelist::breezeTheme {} {
+    variable isAwTheme
+    if {$isAwTheme} {
+	awbreezeTheme 
+    } else {
+	return -code error			;# handled by setThemeDefaults
+    }
+}
+
 #------------------------------------------------------------------------------
 # tablelist::clamTheme
 #------------------------------------------------------------------------------
@@ -387,7 +652,7 @@ proc tablelist::clamTheme {} {
 	-labelborderwidth	2 \
 	-labelpady		3 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
+	-arrowstyle		[defaultX11ArrowStyle] \
 	-treestyle		gtk \
     ]
 }
@@ -434,32 +699,37 @@ proc tablelist::classicTheme {} {
 # tablelist::clearlooksTheme
 #------------------------------------------------------------------------------
 proc tablelist::clearlooksTheme {} {
-    variable themeDefaults
-    array set themeDefaults [list \
-	-background		white \
-	-foreground		black \
-	-disabledforeground	#b5b3ac \
-	-stripebackground	"" \
-	-selectbackground	#71869e \
-	-selectforeground	#ffffff \
-	-selectborderwidth	0 \
-	-font			TkTextFont \
-	-labelbackground	#efeae6 \
-	-labeldeactivatedBg	#efeae6 \
-	-labeldisabledBg	#eee9e4 \
-	-labelactiveBg		#f4f2ee \
-	-labelpressedBg		#d4cfca \
-	-labelforeground	black \
-	-labeldisabledFg	#b5b3ac \
-	-labelactiveFg		black \
-	-labelpressedFg		black \
-	-labelfont		TkDefaultFont \
-	-labelborderwidth	0 \
-	-labelpady		1 \
-	-arrowcolor		black \
-	-arrowstyle		flatAngle9x6 \
-	-treestyle		gtk \
-    ]
+    variable isAwTheme
+    if {$isAwTheme} {
+	awclearlooksTheme 
+    } else {
+	variable themeDefaults
+	array set themeDefaults [list \
+	    -background		white \
+	    -foreground		black \
+	    -disabledforeground	#b5b3ac \
+	    -stripebackground	"" \
+	    -selectbackground	#71869e \
+	    -selectforeground	#ffffff \
+	    -selectborderwidth	0 \
+	    -font		TkTextFont \
+	    -labelbackground	#efeae6 \
+	    -labeldeactivatedBg	#efeae6 \
+	    -labeldisabledBg	#eee9e4 \
+	    -labelactiveBg	#f4f2ee \
+	    -labelpressedBg	#d4cfca \
+	    -labelforeground	black \
+	    -labeldisabledFg	#b5b3ac \
+	    -labelactiveFg	black \
+	    -labelpressedFg	black \
+	    -labelfont		TkDefaultFont \
+	    -labelborderwidth	0 \
+	    -labelpady		1 \
+	    -arrowcolor		black \
+	    -arrowstyle		flatAngle9x6 \
+	    -treestyle		gtk \
+	]
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -489,7 +759,7 @@ proc tablelist::defaultTheme {} {
 	-labelborderwidth	1 \
 	-labelpady		1 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
+	-arrowstyle		[defaultX11ArrowStyle] \
 	-treestyle		gtk \
     ]
 }
@@ -585,7 +855,7 @@ proc tablelist::krocTheme {} {
 	-labelborderwidth	2 \
 	-labelpady		1 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
+	-arrowstyle		[defaultX11ArrowStyle] \
 	-treestyle		gtk \
     ]
 }
@@ -649,7 +919,7 @@ proc tablelist::srivTheme {} {
 	-labelborderwidth	2 \
 	-labelpady		1 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
+	-arrowstyle		[defaultX11ArrowStyle] \
 	-treestyle		gtk \
     ]
 }
@@ -681,7 +951,7 @@ proc tablelist::srivlgTheme {} {
 	-labelborderwidth	2 \
 	-labelpady		1 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
+	-arrowstyle		[defaultX11ArrowStyle] \
 	-treestyle		gtk \
     ]
 }
@@ -713,7 +983,7 @@ proc tablelist::stepTheme {} {
 	-labelborderwidth	2 \
 	-labelpady		1 \
 	-arrowcolor		black \
-	-arrowstyle		flat8x4 \
+	-arrowstyle		[defaultX11ArrowStyle] \
 	-treestyle		gtk \
     ]
 }
@@ -1534,89 +1804,64 @@ proc tablelist::vistaTheme {} {
 	-labelfont		TkDefaultFont \
     ]
 
-    switch [winfo rgb . SystemHighlight] {
-	"13107 39321 65535" {					;# Aero
-	    set selectFg	SystemWindowText
-	    set labelBd		4
-	    set labelPadY	4
+    if {$::tcl_platform(osVersion) >= 10.0} {			;# Win 10
+	set selectBg	#cce8ff
+	set selectFg	SystemWindowText
+	set labelBg	#ffffff
+	set activeBg	#d9ebf9
+	set pressedBg	#bcdcf4
+	set labelBd	4
+	set labelPadY	4
+	set arrowColor	#595959
+	set arrowStyle	flatAngle[defaultWinArrowSize]
+	set treeStyle	win10
 
-	    variable scalingpct
-	    if {$::tcl_platform(osVersion) < 6.2} {		;# Win Vista/7
-		set labelBg	#ffffff
-		set activeBg	#e3f7ff
-		set pressedBg	#Bce4f9
-		set arrowColor	#569bc0
+    } elseif {[string compare [winfo rgb . SystemHighlight] \
+			      "13107 39321 65535"] == 0} {	;# Aero
+	set selectFg	SystemWindowText
+	set labelBd	4
+	set labelPadY	4
 
-		switch $scalingpct {
-		    100 { set arrowStyle	photo7x4 }
-		    125 { set arrowStyle	photo9x5 }
-		    150 { set arrowStyle	photo11x6 }
-		    200 { set arrowStyle	photo15x8 }
-		}
-	    } elseif {$::tcl_platform(osVersion) < 10.0} {	;# Win 8
-		set labelBg	#fcfcfc
-		set activeBg	#f4f9ff
-		set pressedBg	#f9fafb
-		set arrowColor	#569bc0
-
-		switch $scalingpct {
-		    100 { set arrowStyle	photo7x4 }
-		    125 { set arrowStyle	photo9x5 }
-		    150 { set arrowStyle	photo11x6 }
-		    200 { set arrowStyle	photo15x8 }
-		}
-	    } else {						;# Win 10
-		set labelBg	#ffffff
-		set activeBg	#d9ebf9
-		set pressedBg	#bcdcf4
-		set arrowColor	#595959
-
-		switch $scalingpct {
-		    100 { set arrowStyle	flatAngle7x4 }
-		    125 { set arrowStyle	flatAngle9x5 }
-		    150 { set arrowStyle	flatAngle11x6 }
-		    200 { set arrowStyle	flatAngle15x8 }
-		}
-	    }
-
-	    if {$::tcl_platform(osVersion) == 6.0} {		;# Win Vista
-		set selectBg	#d8effb
-		set treeStyle	vistaAero
-	    } elseif {$::tcl_platform(osVersion) == 6.1} {	;# Win 7
-		set selectBg	#cee2fc
-		set treeStyle	win7Aero
-	    } elseif {$::tcl_platform(osVersion) < 10.0} {	;# Win 8
-		set selectBg	#cbe8f6
-		set treeStyle	win7Aero
-	    } else {						;# Win 10
-		set selectBg	#cce8ff
-		set treeStyle	win10
-	    }
+	if {$::tcl_platform(osVersion) < 6.2} {			;# Win Vista/7
+	    set labelBg		#ffffff
+	    set activeBg	#e3f7ff
+	    set pressedBg	#Bce4f9
+	    set arrowColor	#569bc0
+	    set arrowStyle	photo[defaultWinArrowSize]
+	} else {						;# Win 8
+	    set labelBg		#fcfcfc
+	    set activeBg	#f4f9ff
+	    set pressedBg	#f9fafb
+	    set arrowColor	#569bc0
+	    set arrowStyle	photo[defaultWinArrowSize]
 	}
 
-	default {						;# Win Classic
-	    set selectBg	SystemHighlight
-	    set selectFg	SystemHighlightText
-	    set labelBg		SystemButtonFace
-	    set activeBg	SystemButtonFace
-	    set pressedBg	SystemButtonFace
-	    set labelBd		2
-	    set labelPadY	0
-	    set arrowColor	SystemButtonShadow
+	if {$::tcl_platform(osVersion) == 6.0} {		;# Win Vista
+	    set selectBg	#d8effb
+	    set treeStyle	vistaAero
+	} elseif {$::tcl_platform(osVersion) == 6.1} {		;# Win 7
+	    set selectBg	#cee2fc
+	    set treeStyle	win7Aero
+	} else {						;# Win 8
+	    set selectBg	#cbe8f6
+	    set treeStyle	win7Aero
+	}
 
-	    variable scalingpct
-	    switch $scalingpct {
-		100 { set arrowStyle	flat7x4 }
-		125 { set arrowStyle	flat9x5 }
-		150 { set arrowStyle	flat11x6 }
-		200 { set arrowStyle	flat15x8 }
-	    }
+    } else {							;# Win Classic
+	set selectBg	SystemHighlight
+	set selectFg	SystemHighlightText
+	set labelBg	SystemButtonFace
+	set activeBg	SystemButtonFace
+	set pressedBg	SystemButtonFace
+	set labelBd	2
+	set labelPadY	0
+	set arrowColor	SystemButtonShadow
+	set arrowStyle	flat[defaultWinArrowSize]
 
-	    if {$::tcl_platform(osVersion) == 6.0} {		;# Win Vista
-		set treeStyle	vistaClassic
-	    } else {						;# Win 7/8
-		set treeStyle	win7Classic
-	    }
+	if {$::tcl_platform(osVersion) == 6.0} {		;# Win Vista
+	    set treeStyle	vistaClassic
+	} else {						;# Win 7/8
+	    set treeStyle	win7Classic
 	}
     }
 
@@ -1672,32 +1917,37 @@ proc tablelist::winnativeTheme {} {
 # tablelist::winxpblueTheme
 #------------------------------------------------------------------------------
 proc tablelist::winxpblueTheme {} {
-    variable themeDefaults
-    array set themeDefaults [list \
-	-background		white \
-	-foreground		black \
-	-disabledforeground	#565248 \
-	-stripebackground	"" \
-	-selectbackground	#4a6984 \
-	-selectforeground	#ffffff \
-	-selectborderwidth	0 \
-	-font			TkTextFont \
-	-labelbackground	#ece9d8 \
-	-labeldeactivatedBg	#ece9d8 \
-	-labeldisabledBg	#e3e1dd \
-	-labelactiveBg		#c1d2ee \
-	-labelpressedBg		#bab5ab \
-	-labelforeground	black \
-	-labeldisabledFg	#565248 \
-	-labelactiveFg		black \
-	-labelpressedFg		black \
-	-labelfont		TkDefaultFont \
-	-labelborderwidth	2 \
-	-labelpady		1 \
-	-arrowcolor		#4d6185 \
-	-arrowstyle		flat7x4 \
-	-treestyle		winxpBlue \
-    ]
+    variable isAwTheme
+    if {$isAwTheme} {
+	awwinxpblueTheme 
+    } else {
+	variable themeDefaults
+	array set themeDefaults [list \
+	    -background		white \
+	    -foreground		black \
+	    -disabledforeground	#565248 \
+	    -stripebackground	"" \
+	    -selectbackground	#4a6984 \
+	    -selectforeground	#ffffff \
+	    -selectborderwidth	0 \
+	    -font		TkTextFont \
+	    -labelbackground	#ece9d8 \
+	    -labeldeactivatedBg	#ece9d8 \
+	    -labeldisabledBg	#e3e1dd \
+	    -labelactiveBg	#c1d2ee \
+	    -labelpressedBg	#bab5ab \
+	    -labelforeground	black \
+	    -labeldisabledFg	#565248 \
+	    -labelactiveFg	black \
+	    -labelpressedFg	black \
+	    -labelfont		TkDefaultFont \
+	    -labelborderwidth	2 \
+	    -labelpady		1 \
+	    -arrowcolor		#4d6185 \
+	    -arrowstyle		flat7x4 \
+	    -treestyle		winxpBlue \
+	]
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -1720,147 +1970,127 @@ proc tablelist::xpnativeTheme {} {
 	-labelfont		TkDefaultFont \
     ]
 
-    switch [winfo rgb . SystemHighlight] {
-	"12593 27242 50629" {					;# Win XP Blue
-	    set xpStyle		1
-	    set selectBg	SystemHighlight
-	    set selectFg	SystemHighlightText
-	    set labelBg		#ebeadb
-	    set activeBg	#faf8f3
-	    set pressedBg	#dedfd8
-	    set labelBd		4
-	    set labelPadY	4
-	    set arrowColor	#aca899
-	    set arrowStyle	flat9x5
-	    set treeStyle	winxpBlue
+    if {$::tcl_platform(osVersion) >= 10.0} {			;# Win 10
+	set xpStyle	0
+	set selectBg	#cce8ff
+	set selectFg	SystemWindowText
+	set labelBg	#ffffff
+	set activeBg	#d9ebf9
+	set pressedBg	#bcdcf4
+	set labelBd	4
+	set labelPadY	4
+	set arrowColor	#595959
+	set arrowStyle	flatAngle[defaultWinArrowSize]
+	set treeStyle	win10
 
-	    if {[info exists ::tile::version] &&
-		[string compare $::tile::version 0.7] < 0} {
-		set labelBd 0
-	    }
-	}
+    } else {
+	switch [winfo rgb . SystemHighlight] {
+	    "12593 27242 50629" {				;# Win XP Blue
+		set xpStyle	1
+		set selectBg	SystemHighlight
+		set selectFg	SystemHighlightText
+		set labelBg	#ebeadb
+		set activeBg	#faf8f3
+		set pressedBg	#dedfd8
+		set labelBd	4
+		set labelPadY	4
+		set arrowColor	#aca899
+		set arrowStyle	flat9x5
+		set treeStyle	winxpBlue
 
-	"37779 41120 28784" {					;# Win XP Olive
-	    set xpStyle		1
-	    set selectBg	SystemHighlight
-	    set selectFg	SystemHighlightText
-	    set labelBg		#ebeadb
-	    set activeBg	#faf8f3
-	    set pressedBg	#dedfd8
-	    set labelBd		4
-	    set labelPadY	4
-	    set arrowColor	#aca899
-	    set arrowStyle	flat9x5
-	    set treeStyle	winxpOlive
-
-	    if {[info exists ::tile::version] &&
-		[string compare $::tile::version 0.7] < 0} {
-		set labelBd 0
-	    }
-	}
-
-	"45746 46260 49087" {					;# Win XP Silver
-	    set xpStyle		1
-	    set selectBg	SystemHighlight
-	    set selectFg	SystemHighlightText
-	    set labelBg		#f9fafd
-	    set activeBg	#fefefe
-	    set pressedBg	#ececf3
-	    set labelBd		4
-	    set labelPadY	4
-	    set arrowColor	#aca899
-	    set arrowStyle	flat9x5
-	    set treeStyle	winxpSilver
-
-	    if {[info exists ::tile::version] &&
-		[string compare $::tile::version 0.7] < 0} {
-		set labelBd 0
-	    }
-	}
-
-	"13107 39321 65535" {					;# Aero
-	    set selectFg	SystemWindowText
-	    set xpStyle		0
-	    set labelBd		4
-	    set labelPadY	4
-
-	    variable scalingpct
-	    if {$::tcl_platform(osVersion) < 6.2} {		;# Win Vista/7
-		set labelBg	#ffffff
-		set activeBg	#e3f7ff
-		set pressedBg	#Bce4f9
-		set arrowColor	#569bc0
-
-		switch $scalingpct {
-		    100 { set arrowStyle	photo7x4 }
-		    125 { set arrowStyle	photo9x5 }
-		    150 { set arrowStyle	photo11x6 }
-		    200 { set arrowStyle	photo15x8 }
-		}
-	    } elseif {$::tcl_platform(osVersion) < 10.0} {	;# Win 8
-		set labelBg	#fcfcfc
-		set activeBg	#f4f9ff
-		set pressedBg	#f9fafb
-		set arrowColor	#569bc0
-
-		switch $scalingpct {
-		    100 { set arrowStyle	photo7x4 }
-		    125 { set arrowStyle	photo9x5 }
-		    150 { set arrowStyle	photo11x6 }
-		    200 { set arrowStyle	photo15x8 }
-		}
-	    } else {						;# Win 10
-		set labelBg	#ffffff
-		set activeBg	#d9ebf9
-		set pressedBg	#bcdcf4
-		set arrowColor	#595959
-
-		switch $scalingpct {
-		    100 { set arrowStyle	flatAngle7x4 }
-		    125 { set arrowStyle	flatAngle9x5 }
-		    150 { set arrowStyle	flatAngle11x6 }
-		    200 { set arrowStyle	flatAngle15x8 }
+		if {[info exists ::tile::version] &&
+		    [string compare $::tile::version 0.7] < 0} {
+		    set labelBd 0
 		}
 	    }
 
-	    if {$::tcl_platform(osVersion) == 6.0} {		;# Win Vista
-		set selectBg	#d8effb
-		set treeStyle	vistaAero
-	    } elseif {$::tcl_platform(osVersion) == 6.1} {	;# Win 7
-		set selectBg	#cee2fc
-		set treeStyle	win7Aero
-	    } elseif {$::tcl_platform(osVersion) < 10.0} {	;# Win 8
-		set selectBg	#cbe8f6
-		set treeStyle	win7Aero
-	    } else {						;# Win 10
-		set selectBg	#cce8ff
-		set treeStyle	win10
-	    }
-	}
+	    "37779 41120 28784" {				;# Win XP Olive
+		set xpStyle	1
+		set selectBg	SystemHighlight
+		set selectFg	SystemHighlightText
+		set labelBg	#ebeadb
+		set activeBg	#faf8f3
+		set pressedBg	#dedfd8
+		set labelBd	4
+		set labelPadY	4
+		set arrowColor	#aca899
+		set arrowStyle	flat9x5
+		set treeStyle	winxpOlive
 
-	default {						;# Win Classic
-	    set selectBg	SystemHighlight
-	    set selectFg	SystemHighlightText
-	    set xpStyle		0
-	    set labelBg		SystemButtonFace
-	    set activeBg	SystemButtonFace
-	    set pressedBg	SystemButtonFace
-	    set labelBd		2
-	    set labelPadY	0
-	    set arrowColor	SystemButtonShadow
-
-	    variable scalingpct
-	    switch $scalingpct {
-		100 { set arrowStyle	flat7x4 }
-		125 { set arrowStyle	flat9x5 }
-		150 { set arrowStyle	flat11x6 }
-		200 { set arrowStyle	flat15x8 }
+		if {[info exists ::tile::version] &&
+		    [string compare $::tile::version 0.7] < 0} {
+		    set labelBd 0
+		}
 	    }
 
-	    if {$::tcl_platform(osVersion) == 6.0} {		;# Win Vista
-		set treeStyle	vistaClassic
-	    } else {						;# Win 7/8
-		set treeStyle	win7Classic
+	    "45746 46260 49087" {				;# Win XP Silver
+		set xpStyle	1
+		set selectBg	SystemHighlight
+		set selectFg	SystemHighlightText
+		set labelBg	#f9fafd
+		set activeBg	#fefefe
+		set pressedBg	#ececf3
+		set labelBd	4
+		set labelPadY	4
+		set arrowColor	#aca899
+		set arrowStyle	flat9x5
+		set treeStyle	winxpSilver
+
+		if {[info exists ::tile::version] &&
+		    [string compare $::tile::version 0.7] < 0} {
+		    set labelBd 0
+		}
+	    }
+
+	    "13107 39321 65535" {				;# Aero
+		set xpStyle	0
+		set selectFg	SystemWindowText
+		set labelBd	4
+		set labelPadY	4
+
+		if {$::tcl_platform(osVersion) < 6.2} {		;# Win Vista/7
+		    set labelBg	#ffffff
+		    set activeBg	#e3f7ff
+		    set pressedBg	#Bce4f9
+		    set arrowColor	#569bc0
+		    set arrowStyle	photo[defaultWinArrowSize]
+		} else {					;# Win 8
+		    set labelBg	#fcfcfc
+		    set activeBg	#f4f9ff
+		    set pressedBg	#f9fafb
+		    set arrowColor	#569bc0
+		    set arrowStyle	photo[defaultWinArrowSize]
+		}
+
+		if {$::tcl_platform(osVersion) == 6.0} {	;# Win Vista
+		    set selectBg	#d8effb
+		    set treeStyle	vistaAero
+		} elseif {$::tcl_platform(osVersion) == 6.1} {	;# Win 7
+		    set selectBg	#cee2fc
+		    set treeStyle	win7Aero
+		} else {					;# Win 8
+		    set selectBg	#cbe8f6
+		    set treeStyle	win7Aero
+		}
+	    }
+
+	    default {						;# Win Classic
+		set xpStyle	0
+		set selectBg	SystemHighlight
+		set selectFg	SystemHighlightText
+		set labelBg	SystemButtonFace
+		set activeBg	SystemButtonFace
+		set pressedBg	SystemButtonFace
+		set labelBd	2
+		set labelPadY	0
+		set arrowColor	SystemButtonShadow
+		set arrowStyle	flat[defaultWinArrowSize]
+
+		if {$::tcl_platform(osVersion) == 6.0} {	;# Win Vista
+		    set treeStyle	vistaClassic
+		} else {					;# Win 7/8
+		    set treeStyle	win7Classic
+		}
 	    }
 	}
     }
@@ -1881,93 +2111,37 @@ proc tablelist::xpnativeTheme {} {
     ]
 }
 
-#
-# Private procedures performing RGB <-> HSV conversions
-# =====================================================
-#
-
 #------------------------------------------------------------------------------
-# tablelist::rgb2hsv
-#
-# Converts the specified RGB value to HSV.  The arguments are assumed to be
-# integers in the interval [0, 65535].  The return value is a list of the form
-# {h s v}, where h in [0.0, 360.0) and s, v in [0.0, 1.0].
+# tablelist::awTheme
 #------------------------------------------------------------------------------
-proc tablelist::rgb2hsv {r g b} {
-    set r [expr {$r/65535.0}]
-    set g [expr {$g/65535.0}]
-    set b [expr {$b/65535.0}]
-
-    #
-    # Compute the value component
-    #
-    set sortedList [lsort -real [list $r $g $b]]
-    set v [lindex $sortedList end]
-    set dist [expr {$v - [lindex $sortedList 0]}]
-
-    #
-    # Compute the saturation component
-    #
-    if {$v == 0.0} {
-	set s 0.0
-    } else {
-	set s [expr {$dist/$v}]
-    }
-
-    #
-    # Compute the hue component
-    #
-    if {$s == 0.0} {
-	set h 0.0
-    } else {
-	set rc [expr {($v - $r)/$dist}]
-	set gc [expr {($v - $g)/$dist}]
-	set bc [expr {($v - $b)/$dist}]
-
-	if {$v == $r} {
-	    set h [expr {$bc - $gc}]
-	} elseif {$v == $g} {
-	    set h [expr {2 + $rc - $bc}]
-	} else {
-	    set h [expr {4 + $gc - $rc}]
-	}
-	set h [expr {$h*60}]
-	if {$h < 0.0} {
-	    set h [expr {$h + 360.0}]
-	} elseif {$h >= 360.0} {
-	    set h 0.0
-	}
-    }
-
-    return [list $h $s $v]
-}
-
-#------------------------------------------------------------------------------
-# tablelist::hsv2rgb
-#
-# Converts the specified HSV value to RGB.  The arguments are assumed to fulfil
-# the conditions: h in [0.0, 360.0) and s, v in [0.0, 1.0].  The return value
-# is a list of the form {r g b}, where r, g, and b are integers in the interval
-# [0, 65535].
-#------------------------------------------------------------------------------
-proc tablelist::hsv2rgb {h s v} {
-    set h [expr {$h/60.0}]
-    set f [expr {$h - floor($h)}]
-
-    set p1 [expr {round(65535.0*$v*(1 - $s))}]
-    set p2 [expr {round(65535.0*$v*(1 - $s*$f))}]
-    set p3 [expr {round(65535.0*$v*(1 - $s*(1 - $f)))}]
-
-    set v  [expr {round(65535.0*$v)}]
-
-    switch [expr {int($h)}] {
-	0 { return [list $v  $p3 $p1] }
-	1 { return [list $p2 $v  $p1] }
-	2 { return [list $p1 $v  $p3] }
-	3 { return [list $p1 $p2 $v ] }
-	4 { return [list $p3 $p1 $v ] }
-	5 { return [list $v  $p1 $p2] }
-    }
+proc tablelist::awTheme {} {
+    set bg [styleConfig . -background]
+    set fg [styleConfig . -foreground]
+    set disabledFg [lindex [style map . -foreground] 1]
+    variable themeDefaults
+    array set themeDefaults [list \
+	-background		$bg \
+	-foreground		$fg \
+	-disabledforeground	$disabledFg \
+	-selectbackground	[styleConfig . -selectbackground] \
+	-selectforeground	[styleConfig . -selectforeground] \
+	-selectborderwidth	[styleConfig . -selectborderwidth] \
+	-font			TkTextFont \
+	-labelbackground	$bg \
+	-labeldeactivatedBg	$bg \
+	-labeldisabledBg	$bg \
+	-labelactiveBg		$bg \
+	-labelpressedBg		$bg \
+	-labelforeground	$fg \
+	-labeldisabledFg	$disabledFg \
+	-labelactiveFg		$fg \
+	-labelpressedFg		$fg \
+	-labelfont		TkDefaultFont \
+	-labelborderwidth	2 \
+	-labelpady		2 \
+	-arrowcolor		$fg \
+	-arrowstyle		[defaultX11ArrowStyle] \
+    ]
 }
 
 #
